options nocenter nodate nonumber linesize=120 pagesize=65 formdlim=' '; title;

*This is the New York Crime Data for Chapter 2;
data in; 
  set "c:\data\nypa95_listwise"; 
  where pa = 0; 
  cnt = _N_; 
  keep janfeb95 marapr95 mayjun95 jlyaug95 pov12590 popden90 stplc2; 
run;

title1 "Running the OLSREG4 Macro reproduces the OLS results";
title2 "presented in tables 2.2 and 2.3";
* ALPHA HAT, BETA HAT ;
proc sort data = in; 
  by stplc2; 
run;

%macro olsreg4(indd,t1var,t2var,t3var,t4var,linout,quadout);
  data olsreg;
    set &indd;
    if &t1var ne . and &t2var ne . and &t3var ne . and &t4var ne .;

    keep stplc2 time timesq regvar;

    time   = 0;
    timesq = 0;
    regvar = &t1var;
    output;
    time   = 1;
    timesq = 1;
    regvar = &t2var;
    output;
    time   = 2;
    timesq = 4;
    regvar = &t3var;
    output;
    time   = 3;
    timesq = 9;
    regvar = &t4var;
    output;
  run;

  proc reg data = olsreg outest = olsdat rsquare;
    by stplc2;
    model regvar = time / noprint covb;
  run;

  proc reg data = olsreg; 
    model regvar = time / covb ;
  run;


  data olsdat4 (keep = stplc2 intercept linear rsquared read_res);
    set olsdat;
    read_res=(_rmse_*2);
    rename time  = linear
           _rsq_ = rsquared;
  run;

  data &linout;
    set olsdat4;
  run;

  proc reg data = olsreg outest = olsdat2 rsquare;
    by stplc2;
    model regvar = time timesq / noprint covb;
  run;

  data olsdat2 (keep = stplc2 intercept linear quad rsquared read_res);
    set olsdat2;
    read_res=(_rmse_*2);
    rename time   = linear
           timesq = quad
           _rsq_  = rsquared;
  run;

  data &quadout;
    set olsdat2;
  run;
%mend olsreg4;
%olsreg4(in,janfeb95,marapr95,mayjun95,jlyaug95,readlin,readquad);



title1 "Means and variances presented in the text can be reproduced by";
title2 "the following code and proc univariate syntax";
Data olsdat_adj;
  set olsdat4;
  varB=.039;
  varA=.598;
  varE=.0982;
  T=4;
  Sum_Lambda_SQ=14;
  varLambda=5;
  Psi_alpha=varA-(varE*Sum_Lambda_SQ/(varLAmbda*T));
  Psi_beta=varB-(varE/varLAmbda);
run;

proc univariate data = olsdat_adj plots;
  var intercept linear read_res Psi_alpha Psi_beta;
  title 'Univariate Statistics, OLS Regression Estimates';
  title2 "&dataset";
  *%labelit;
run;

*  Proc means below produce alternate computations ;
*  of Mu_alpha and Mu_beta under ommission of      ;
*  outliers described in the book                  ;
proc means data = olsdat4; 
  var intercept; 
  where intercept < 8.5; 
run;

proc means data = olsdat4; 
  var linear; 
  where linear > -1.1; 
run;

title1 "IML statements and regression produce OLS trajectory" ;
title2 "estimates for first case in sample as presented in table 2.1";
proc iml;

  y1i = {4.00885,
         4.00885,
         5.78562,
         4.69287};

  lambda = {1 0,
            1 1,
            1 2,
            1 3};
  y1i = y1i || lambda;
  CREATE a1 FROM y1i;
  APPEND FROM y1i;
quit;

proc reg data = a1; 
  model  col1 = col3; 
run;
quit;

title1 "Reproduces ML results presented in table 2.3" ;
title2 "New York-  Linear model- equal theta epps";
proc calis data=in ucov aug print; 
  var janfeb95 marapr95 mayjun95 jlyaug95 ;
  lineqs
    janfeb95= 1f1 + 0f2 + e1,
    marapr95= 1f1 + 1f2 + e2,
    mayjun95= 1f1 + 2f2 + e3,
    jlyaug95= 1f1 + 3f2 + e4,
    f1 = al1 intercept + d1,
    f2 = al2 intercept + d2;
  std
    e1-e4 = th1 th1 th1 th1,
    d1-d2 = ph11 ph22;
  cov
    d1 d2 = ph21;
run;
